package cs2114.demo.circles;

import android.graphics.RectF;
import android.view.MotionEvent;
import sofia.app.OptionsMenu;
import sofia.app.ShapeScreen;
import sofia.graphics.Color;
import sofia.graphics.OvalShape;

// -------------------------------------------------------------------------
/**
 * A demo that shows how you can use touch input to draw shapes on the screen.
 *
 * @author  Tony Allevato
 * @version 2012.10.03
 */
@OptionsMenu
public class CirclesScreen extends ShapeScreen
{
    //~ Fields ................................................................

    private Color currentColor = Color.red;
    private OvalShape currentShape;


    //~ Public methods ........................................................

    // ----------------------------------------------------------
    /**
     * Called when the user touches his or her finger down on the screen.
     *
     * @param event a MotionEvent that describes the event that occurred, such
     *     as its position
     */
    public void onTouchDown(MotionEvent event)
    {
        // Get the x- and y-coordinates of the touch from the event object.

        float x = event.getX();
        float y = event.getY();

        currentShape = new OvalShape(new RectF(x, y, x + 1, y + 1));
        currentShape.setColor(currentColor);
        currentShape.setFilled(true);

        // Set the shape's alpha level low so that we can see what is under
        // the shape while we're dragging.

        currentShape.setAlpha(48);

        // Add the shape to the screen so that it will be drawn.

        add(currentShape);
    }


    // ----------------------------------------------------------
    /**
     * Called rapidly and repeatedly when the user's finger is dragged around
     * the screen.
     *
     * @param event a MotionEvent that describes the event that occurred, such
     *     as its position
     */
    public void onTouchMove(MotionEvent event)
    {
        // To change the size of the shape we need to get the bounds (which
        // returns a RectF object), change its right and bottom fields, and
        // then set them back.

        RectF bounds = currentShape.getBounds();
        bounds.right = event.getX();
        bounds.bottom = event.getY();
        currentShape.setBounds(bounds);
    }


    // ----------------------------------------------------------
    /**
     * Called when the user lifts his or her finger off the screen.
     *
     * @param event a MotionEvent that describes the event that occurred, such
     *     as its position
     */
    public void onTouchUp(MotionEvent event)
    {
        // Run an 500 millisecond (1/2 second) animation that fades the shape
        // in from its starting alpha value of 48 to a final alpha value of
        // 160 (so it is still a little transparent, but not as much).

        currentShape.animate(500).alpha(160).play();
    }


    // ----------------------------------------------------------
    /**
     * Called when the "Red" option is selected in the menu.
     */
    public void redClicked()
    {
        currentColor = Color.red;
    }


    // ----------------------------------------------------------
    /**
     * Called when the "Green" option is selected in the menu.
     */
    public void greenClicked()
    {
        currentColor = Color.green;
    }


    // ----------------------------------------------------------
    /**
     * Called when the "Blue" option is selected in the menu.
     */
    public void blueClicked()
    {
        currentColor = Color.blue;
    }


    // ----------------------------------------------------------
    /**
     * Called when the "Yellow" option is selected in the menu.
     */
    public void yellowClicked()
    {
        currentColor = Color.yellow;
    }
}
