package cs2114.tipcalculator;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import java.lang.reflect.Field;

import sofia.app.Screen;
import student.AndroidTestCase;
import student.testingsupport.ReflectionSupport;
import android.widget.EditText;
import android.widget.RadioButton;
import android.widget.TextView;

// -------------------------------------------------------------------------
/**
 *  Instructor reference tests for Lab 03: Tip Calculator (Sofia version).
 *
 *  @author  Tony Allevato
 *  @version 2011.07.11
 */
public class __TipCalculatorScreenReferenceTest
	extends AndroidTestCase<Screen>
{
	// ----------------------------------------------------------
	/*
	 * This should be moved into the support library. Or, be made automatic
	 * for the Robolectric version of student.AndroidTestCase.
	 */
	@Retention(RetentionPolicy.RUNTIME)
	@Target(ElementType.FIELD)
	@interface AssertInLayout { }


	//~ Fields ................................................................

	@AssertInLayout private EditText amountOfBill;
	@AssertInLayout private RadioButton tip15;
	@AssertInLayout private RadioButton tip18;
	@AssertInLayout private RadioButton tip20;
	@AssertInLayout private TextView tipAmount;

	private Class<?> screenClass;


    //~ Constructors ..........................................................

    // ----------------------------------------------------------
    /**
     * Creates a new __TipCalculatorScreenReferenceTest object to test the
     * student's TipCalculatorScreen class.
     */
    public __TipCalculatorScreenReferenceTest()
    {
        super("cs2114.tipcalculator.TipCalculatorScreen");
    }


    //~ Public methods ........................................................

    // ----------------------------------------------------------
    public void setUp()
    {
    	assertViewsInLayout();

    	screenClass = ReflectionSupport.getClassForName(
    			"cs2114.tipcalculator.TipCalculatorScreen");
    }


    // ----------------------------------------------------------
    public void testRequiredMethods()
    {
    	Class<?> tcKlass = ReflectionSupport.getClassForName(
    			"cs2114.tipcalculator.TipCalculator");
    	
    	ReflectionSupport.getMatchingMethod(screenClass, "initialize");
    	ReflectionSupport.getMatchingMethod(
    			screenClass, "changeWasObserved", tcKlass);
    }


    // ----------------------------------------------------------
    /**
     * Tests that the tip rate is initially set to 15% by default.
     */
    public void testDefaultTipIs15()
    {
    	ReflectionSupport.getMatchingMethod(
    			screenClass, "amountOfBillEditingDone");

    	enterText(amountOfBill, "12.34");

        assertTrue("The text in the tip amount label was not correct when "
        		+ "entering a bill amount and leaving the default tip rate "
        		+ "selected. Did you remember to set the tip rate to 0.15 "
        		+ "in your initialize() method?",
            contains(tipAmount.getText().toString(), "1.85"));
    }
    

    // ----------------------------------------------------------
    /**
     * Tests the 15% tip button.
     */
    public void testTip15()
    {
    	ReflectionSupport.getMatchingMethod(screenClass, "tip15Clicked");
    	ReflectionSupport.getMatchingMethod(
    			screenClass, "amountOfBillEditingDone");

    	enterText(amountOfBill, "12.34");
        click(tip15);

        assertTrue("The text in the tip amount label was not correct when "
        		+ "entering a bill amount and then choosing a 15% tip.",
            contains(tipAmount.getText().toString(), "1.85"));
    }
    

    // ----------------------------------------------------------
    /**
     * Tests the 18% tip button.
     */
    public void testTip18()
    {
    	ReflectionSupport.getMatchingMethod(screenClass, "tip18Clicked");
    	ReflectionSupport.getMatchingMethod(
    			screenClass, "amountOfBillEditingDone");

    	enterText(amountOfBill, "98.67");
        click(tip18);

        assertTrue("The text in the tip amount label was not correct when "
        		+ "entering a bill amount and then choosing an 18% tip.",
            contains(tipAmount.getText().toString(), "17.76"));
    }
    

    // ----------------------------------------------------------
    /**
     * Tests the 20% tip button.
     */
    public void testTip20()
    {
    	ReflectionSupport.getMatchingMethod(screenClass, "tip20Clicked");
    	ReflectionSupport.getMatchingMethod(
    			screenClass, "amountOfBillEditingDone");

    	enterText(amountOfBill, "5.55");
        click(tip20);

        assertTrue("The text in the tip amount label was not correct when "
        		+ "entering a bill amount and then choosing a 20% tip.",
            contains(tipAmount.getText().toString(), "1.11"));
    }
    

    // ----------------------------------------------------------
    /**
     * Tests flipping the order: choosing a tip amount and then entering a
     * dollar amount for the bill.
     */
    public void testAmountAfterTip()
    {
    	ReflectionSupport.getMatchingMethod(screenClass, "tip18Clicked");
    	ReflectionSupport.getMatchingMethod(
    			screenClass, "amountOfBillEditingDone");

    	click(tip18);
        enterText(amountOfBill, "98.67");

        assertTrue("The text in the tip amount label was not correct when "
        		+ "a new bill amount was entered after a tip rate was "
        		+ "selected. Are you correctly handling the EditingDone "
        		+ "event?",
            contains(tipAmount.getText().toString(), "17.76"));
    }
    

    // ----------------------------------------------------------
    /**
     * Verifies that the student's layout had the necessary views and provides
     * meaningful hints if any are missing.
     * 
     * @param ids the ids of widgets to verify
     */
    private void assertViewsInLayout(String... ids)
    {
    	for (Field field : getClass().getDeclaredFields())
    	{
    		if (field.getAnnotation(AssertInLayout.class) != null)
    		{
	    		Object result = null;
	
		    	try
		    	{
		    		result = field.get(this);
		    	}
		    	catch (Exception e)
		    	{
		    		result = null;
		    	}
	    		
	    		assertNotNull("Your layout does not appear to have an "
	    				+ field.getType().getSimpleName() + " widget with the "
	    				+ "id \"" + field.getName() + "\".", result);
    		}
    	}
    }
}
