package cs2114.flashlight;

import java.util.Set;

import sofia.app.ShapeScreen;
import sofia.graphics.Color;
import sofia.graphics.OvalShape;
import sofia.graphics.RectangleShape;
import sofia.graphics.ShapeView;
import student.testingsupport.annotations.Hint;

//-------------------------------------------------------------------------
/**
 * Reference tests for the Flashlight lab.
 * 
 * @author  Tony Allevato
 * @version 2012.10.03
 */
public class __FlashlightScreenReferenceTest
	extends student.AndroidTestCase<ShapeScreen>
{
	//~ Fields ................................................................

	private ShapeView shapeView;
	
	private int viewCenterX;
	private int viewCenterY;


	//~ Constructors ..........................................................

	// ----------------------------------------------------------
	public __FlashlightScreenReferenceTest()
	{
		super("cs2114.flashlight.FlashlightScreen");
	}
	
	
	//~ Methods ...............................................................

	// ----------------------------------------------------------
	public void setUp()
	{
		shapeView = getScreen().getShapeView();
		viewCenterX = shapeView.getWidth() / 2;
		viewCenterY = shapeView.getHeight() / 2;
	}
	
	
	// ----------------------------------------------------------
	@Hint("There appears to be a problem when looking for the colored boxes.")
	public void testHasBoxes()
	{
		RectangleShape whiteBox = shapeView.getShapeAt(
				1, 1, RectangleShape.class);
		assertNotNull("There should be a box in the top-left corner of the "
				+ "screen.", whiteBox);
		assertEquals("The box at the top-left corner of the screen should be "
				+ "white.", Color.white, whiteBox.getColor());

		RectangleShape redBox = shapeView.getShapeAt(
				getScreen().getWidth() - 2, 1, RectangleShape.class);
		assertNotNull("There should be a box in the top-right corner of the "
				+ "screen.", redBox);
		assertEquals("The box at the top-right corner of the screen should be "
				+ "red.", Color.red, redBox.getColor());

		RectangleShape greenBox = shapeView.getShapeAt(
				1, getScreen().getHeight() - 2, RectangleShape.class);
		assertNotNull("There should be a box in the bottom-left corner of the "
				+ "screen.", greenBox);
		assertEquals("The box at the bottom-left corner of the screen should be "
				+ "green.", Color.green, greenBox.getColor());

		RectangleShape blueBox = shapeView.getShapeAt(
				getScreen().getWidth() - 2, getScreen().getHeight() - 2,
				RectangleShape.class);
		assertNotNull("There should be a box in the bottom-right corner of the "
				+ "screen.", blueBox);
		assertEquals("The box at the bottom-right corner of the screen should be "
				+ "blue.", Color.blue, blueBox.getColor());
	}


	// ----------------------------------------------------------
	@Hint("There appears to be a problem when the view is touched.")
	public void testTouchDown()
	{
		touchDown(shapeView, viewCenterX, viewCenterY);

		Set<OvalShape> shapes = shapeView.getShapes(OvalShape.class);
		assertTrue("Touching the view did not create an oval on the screen.",
				shapes.size() != 0);
		assertTrue("Touching the view should only create one oval on the "
				+ "screen.", shapes.size() == 1);
		
		OvalShape shape = shapes.iterator().next();
		assertEquals("Touching the view should create an oval with the same "
				+ "width and height.", shape.getWidth(), shape.getHeight(),
				0.001);

		float cx = shape.getX() + shape.getWidth() / 2;
		float cy = shape.getY() + shape.getHeight() / 2;

		assertEquals("The oval created when touching the view should be "
				+ "centered at the point that was touched.", viewCenterX, cx,
				0.001);
		assertEquals("The oval created when touching the view should be "
				+ "centered at the point that was touched.", viewCenterY, cy,
				0.001);

		assertEquals("The oval created when touching the view should be "
				+ "white.", Color.white, shape.getColor());
	}


	// ----------------------------------------------------------
	@Hint("There appears to be a problem when the view is touched and then "
			+ "the finger is moved.")
	public void testTouchMove()
	{
		float destX = viewCenterX + 5;
		float destY = viewCenterY - 5;

		touchDown(shapeView, viewCenterX, viewCenterY);
		touchMove(destX, destY);
		
		Set<OvalShape> shapes = shapeView.getShapes(OvalShape.class);
		assertTrue("There should still be an oval on the screen when the "
				+ "finger is moved.", shapes.size() != 0);
		assertTrue("Moving the finger should not create more ovals on the "
				+ "screen.", shapes.size() == 1);

		OvalShape shape = shapes.iterator().next();
		assertEquals("After moving the finger, the oval should still have the "
				+ "same width and height.", shape.getWidth(),
				shape.getHeight(), 0.001);

		float cx = shape.getX() + shape.getWidth() / 2;
		float cy = shape.getY() + shape.getHeight() / 2;

		assertEquals("The oval created when touching the view should be "
				+ "centered at the point that was touched.", destX, cx,
				0.001);
		assertEquals("The oval created when touching the view should be "
				+ "centered at the point that was touched.", destY, cy,
				0.001);
	}


	// ----------------------------------------------------------
	@Hint("There appears to be a problem when the view is touched and then "
			+ "released.")
	public void testTouchUp()
	{
		float destX = viewCenterX + 5;
		float destY = viewCenterY - 5;

		touchDown(shapeView, viewCenterX, viewCenterY);
		touchMove(destX, destY);
		touchUp();
		
		Set<OvalShape> shapes = shapeView.getShapes(OvalShape.class);
		assertEquals("There should not be any ovals on the screen after the "
				+ "finger is released.", 0, shapes.size());
	}


	// ----------------------------------------------------------
	@Hint("There appears to be a problem when the light is dragged onto the "
			+ "white box.")
	public void testChangeToWhite()
	{
		touchDown(shapeView, viewCenterX, viewCenterY);
		touchMove(0, 0);
		
		Set<OvalShape> shapes = shapeView.getShapes(OvalShape.class);
		assertTrue("There should still be an oval on the screen when the "
				+ "finger is moved.", shapes.size() != 0);
		assertTrue("Moving the finger should not create more ovals on the "
				+ "screen.", shapes.size() == 1);

		OvalShape shape = shapes.iterator().next();
		assertEquals("The light should change its color to white after it is "
				+ "moved over the white box.", Color.white, shape.getColor());
	}


	// ----------------------------------------------------------
	@Hint("There appears to be a problem when the light is dragged onto the "
			+ "red box.")
	public void testChangeToRed()
	{
		touchDown(shapeView, viewCenterX, viewCenterY);
		touchMove(getScreen().getWidth(), 0);
		
		Set<OvalShape> shapes = shapeView.getShapes(OvalShape.class);
		assertTrue("There should still be an oval on the screen when the "
				+ "finger is moved.", shapes.size() != 0);
		assertTrue("Moving the finger should not create more ovals on the "
				+ "screen.", shapes.size() == 1);

		OvalShape shape = shapes.iterator().next();
		assertEquals("The light should change its color to red after it is "
				+ "moved over the red box.", Color.red, shape.getColor());
	}


	// ----------------------------------------------------------
	@Hint("There appears to be a problem when the light is dragged onto the "
			+ "green box.")
	public void testChangeToGreen()
	{
		touchDown(shapeView, viewCenterX, viewCenterY);
		touchMove(0, getScreen().getHeight());
		
		Set<OvalShape> shapes = shapeView.getShapes(OvalShape.class);
		assertTrue("There should still be an oval on the screen when the "
				+ "finger is moved.", shapes.size() != 0);
		assertTrue("Moving the finger should not create more ovals on the "
				+ "screen.", shapes.size() == 1);

		OvalShape shape = shapes.iterator().next();
		assertEquals("The light should change its color to green after it is "
				+ "moved over the green box.", Color.green, shape.getColor());
	}


	// ----------------------------------------------------------
	@Hint("There appears to be a problem when the light is dragged onto the "
			+ "blue box.")
	public void testChangeToBlue()
	{
		touchDown(shapeView, viewCenterX, viewCenterY);
		touchMove(getScreen().getWidth(), getScreen().getHeight());
		
		Set<OvalShape> shapes = shapeView.getShapes(OvalShape.class);
		assertTrue("There should still be an oval on the screen when the "
				+ "finger is moved.", shapes.size() != 0);
		assertTrue("Moving the finger should not create more ovals on the "
				+ "screen.", shapes.size() == 1);

		OvalShape shape = shapes.iterator().next();
		assertEquals("The light should change its color to blue after it is "
				+ "moved over the blue box.", Color.blue, shape.getColor());
	}
}
